/*
 * ar4c-dcdc - storage.c
 *
 * Copyright (C) 2014  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <storage.h>
#include <watchdog.h>
#include <ltypes.h>
#include <comm.h>

#define CONFIG_BASE		0x0000

#define CONFIG_COPIES	2

u16 config_read(unsigned char reg)
{
	unsigned int addr = CONFIG_BASE + 8 * reg;
	unsigned char i = 0;
	u16 x;
	u16 csum;

	do {
		x = ee_read(addr + 0) << 8;
		x |= ee_read(addr + 1);
		csum = ee_read(addr + 2) << 8;
		csum |= ee_read(addr + 3);
		wdt_reset();
		if (csum == ~x)
			return x;
		addr += 4;
		i++;
	} while (i < CONFIG_COPIES);
	return 0;
}

void config_write(u16 x, unsigned char reg)
{
	unsigned int addr = CONFIG_BASE + 8 * reg;
	unsigned char i = 0;
	u16 csum;

	csum = ~x;
	do {
		ee_write(x >> 8, addr + 0);
		ee_write(x & 0xff, addr + 1);
		ee_write(csum >> 8, addr + 2);
		ee_write(csum & 0xff, addr + 3);
		wdt_reset();
		addr += 4;
		i++;
	} while (i < CONFIG_COPIES);
}
